unit Unit1;

// - VCC -
// Main window of JTAG CDC Loader
// For Spartan-3(E) and Spartan-6 FPGAs
// - 2010 -
// - 2013 -

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    btnGetJTAGChain: TButton;
    btnProgramSelectedDevice: TButton;
    tmrStartup: TTimer;
    lstLog: TListBox;
    btnAbortProgramming: TButton;
    prbProgramming: TProgressBar;
    pnlProgrammingStatus: TPanel;
    lbeDeviceToProgram: TLabeledEdit;
    pnlProgramPos: TPanel;
    cmbComPort: TComboBox;
    StatusBar1: TStatusBar;
    btnBrowseForBitfile: TButton;
    OpenDialog1: TOpenDialog;
    btnBlinkLED: TButton;
    btnGetIDCodes: TButton;
    btnResetPIC: TButton;
    procedure btnGetJTAGChainClick(Sender: TObject);
    procedure tmrStartupTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnProgramSelectedDeviceClick(Sender: TObject);
    procedure btnAbortProgrammingClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmbComPortChange(Sender: TObject);
    procedure btnBrowseForBitfileClick(Sender: TObject);
    procedure btnBlinkLEDClick(Sender: TObject);
    procedure btnGetIDCodesClick(Sender: TObject);
    procedure btnResetPICClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function peek: Boolean; {Peek Message}
    procedure GetJTAGChain;
    procedure ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
    procedure LoadSettingsFromIni;
    procedure SaveSettingsToIni;
  end;

const
  timeout = 2000;


  CMaxDevices = 30; {Maximum number of devices}
  CDBDeviceCount = 19; {Number of items in Device database}

  CDeviceNameArray: array[0..CDBDeviceCount] of AnsiString =
    ('XC3S100E', 'XC3S250E', 'XC3S500E', 'XC3S1200E', 'XC3S1600E',
    'XCF02S', 'XCF04S', 'XCF08P',
    'XC3S400',
    'XC2C64A',
    'XC6SLX4',
    'XC6SLX9',
    'XC6SLX16',
    'XC6SLX25',
    'XC6SLX25T',
    'XC6SLX45',
    'XC6SLX45T',
    'XC6SLX75',
    'XC6SLX75T',

    'Unknown');


var
  Form1: TForm1;
  ChainLength: Byte;
  ReceiveBuffer: array[0..63] of AnsiChar;
  AbortProgrammging: Boolean;

  COMnumber: Byte = 1;
  CurrentFileName: TFileName;

implementation

{$R *.dfm}

uses
  JTAGdataProvider, BitToTibConverter, IniFiles, WinCom;

function TForm1.peek: Boolean;
var
  msg: TMsg;
begin
  Result := False;
  if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then
  begin
    if (msg.message = WM_QUIT) then
      Result := True
    else
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end
  end;
end;

procedure TForm1.GetJTAGChain;
var
  ReadResult: Cardinal;
  i: Byte;
  a: array[1..64] of AnsiChar;
begin
  //SendString(COMnumber, #10);
  a[1] := #10;
  SendChars(COMnumber, a, 64);
  Sleep(100);
  
  repeat
    peek;
    //ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, SizeOf(ReceiveBuffer));
    ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, 63);
  until ReadResult > 0;
  
  ChainLength := Ord(ReceiveBuffer[0]);
  lstLog.Items.Add('');
  lstLog.Items.Add('ChainLength = ' + IntToStr(ChainLength));

  for i := 1 to ChainLength do
    lstLog.Items.Add(WideString(CDeviceNameArray[Ord(ReceiveBuffer[i])]) + '  Position: ' + IntToStr(i));

  FlushReceiveBuffer(COMnumber);  
end;

procedure TForm1.ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
var
  NoMoreData: Boolean;
  arr: ArrayOf61Chars;
  NumberOfWrittenBytes: Cardinal;
  DataLength: Byte;
  ReadResult: Cardinal;
  a: array[1..64] of AnsiChar;
begin
  OpenTheTibFile(ATibFile);

  //SendString(COMnumber, #20 + Chr(PositionInChain)); {program command + first device}
  a[1] := #20;
  a[2] := AnsiChar(Chr(PositionInChain));
  SendChars(COMnumber, a, 64);
  
  Sleep(1);

  prbProgramming.Position := 0;
  AbortProgrammging := False;
  repeat

    repeat
      peek;
      //ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, SizeOf(ReceiveBuffer));
      ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, 63);
      if ReadResult > 0 then
        if ReceiveBuffer[0] = #25 then   {PIC asks for more data}
        begin
          FlushReceiveBuffer(COMnumber);
          Break;
        end;
    until AbortProgrammging;
    
    ReadFromTibFileAsAStream(@arr);
    DataLength := arr[0];

    if AbortProgrammging then
      arr[0] := arr[0] or 64;

      
    WriteFile(ComData[COMnumber]^.Handle, arr, arr[0] and $3F + 1, NumberOfWrittenBytes, nil);


    NoMoreData := arr[0] and 128 = 128;

    DataLength := DataLength and $7F;

    prbProgramming.Position := prbProgramming.Position + DataLength;
    if Random(4) = 1 then  {do not update every time}
      pnlProgramPos.Caption := 'Program pos: ' + IntToStr(prbProgramming.Position) + ' / ' + IntToStr(prbProgramming.Max);

  until NoMoreData or AbortProgrammging;

  CloseTheTibFile;

  pnlProgramPos.Caption := 'Program pos: ' + IntToStr(prbProgramming.Position) + ' / ' + IntToStr(prbProgramming.Max);
end;

procedure TForm1.btnGetJTAGChainClick(Sender: TObject);
begin
  if not ComIsOpen(COMnumber) then
    Exit;

  GetJTAGChain;
  
  if ChainLength > 0 then
    btnProgramSelectedDevice.Enabled := True;

  lstLog.ItemIndex := lstLog.Count - 1; //Select last line  
end;

procedure TForm1.btnProgramSelectedDeviceClick(Sender: TObject);
var
  TibFileName: TFileName;
  PositionToProgram: Byte;
  BitLength: Integer;
begin
  if not ComIsOpen(COMnumber) then
    Exit;

  PositionToProgram := StrToInt(lbeDeviceToProgram.Text);

  if not FileExists(CurrentFileName) then
    if not OpenDialog1.Execute then
      Exit
    else
    begin
      CurrentFileName := OpenDialog1.FileName;
      StatusBar1.Panels.Items[1].Text := CurrentFileName;
    end;

  pnlProgrammingStatus.Caption := 'Programming';
  pnlProgrammingStatus.Repaint;

  btnGetJTAGChain.Enabled := False;
  btnProgramSelectedDevice.Enabled := False;


  try
    if PositionToProgram > ChainLength then
      MessageBox(Handle, PChar('Position selected exceeds chain length.' + #13#10 + 'Valid positions are in range 1..' + IntToStr(ChainLength)), 'JTAG error', MB_ICONERROR)
    else
    begin
      TibFileName := ConvertBitFileToTibFile(CurrentFileName, BitLength);
      prbProgramming.Max := BitLength;
      ProgramFPGA(TibFileName, PositionToProgram);  {Send the .tib file to microcontroller}
    end;
  finally
    btnGetJTAGChain.Enabled := True;
    btnProgramSelectedDevice.Enabled := True;
  end;

  pnlProgrammingStatus.Caption := 'Finished';
  pnlProgrammingStatus.Repaint;
end;

procedure TForm1.btnResetPICClick(Sender: TObject);
var
  a: array[1..64] of AnsiChar;
begin
  //SendString(COMnumber, #30);
  a[1] := #50;
  SendChars(COMnumber, a, 64);

  Sleep(100);
  FlushReceiveBuffer(COMnumber);
end;

procedure TForm1.btnAbortProgrammingClick(Sender: TObject);
begin
  AbortProgrammging := True;
end;

procedure TForm1.btnBrowseForBitfileClick(Sender: TObject);
begin
  if not OpenDialog1.Execute then
    Exit;
    
  CurrentFileName := OpenDialog1.FileName;
  StatusBar1.Panels.Items[1].Text := CurrentFileName;
end;

procedure TForm1.btnBlinkLEDClick(Sender: TObject);
var
  a: array[1..64] of AnsiChar;
begin
  //SendString(COMnumber, #30);
  a[1] := #30;
  SendChars(COMnumber, a, 64);

  Sleep(100);
  FlushReceiveBuffer(COMnumber);
end;

procedure TForm1.btnGetIDCodesClick(Sender: TObject);
const
  CTimeout: Cardinal = 3000;
var
  tk: Cardinal;
  i: Integer;
  ReadResult: Cardinal;
  a: array[1..64] of AnsiChar;
begin
  //SendString(COMnumber, #40);
  a[1] := #40;
  SendChars(COMnumber, a, 64);
  
  Sleep(100);

  tk := GetTickCount;
  repeat
    peek;
    //ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, SizeOf(ReceiveBuffer));
    ReadResult := ReceiveChars(COMnumber, ReceiveBuffer, 63);
  until (ReadResult > 0) or (GetTickCount - tk >= CTimeout); {wait for PIC to answer}

  if GetTickCount - tk >= CTimeout then
    MessageBox(Handle, 'Can''t get JTAG ID codes.', PChar(Application.Title), MB_ICONERROR)
  else
  begin
    ChainLength := Ord(ReceiveBuffer[0]);
    lstLog.Items.Add('');
    lstLog.Items.Add('ChainLength = ' + IntToStr(ChainLength));

    for i := 1 to ChainLength do
      lstLog.Items.Add('Device ' + IntToStr(i) + '  ID = ' + IntToStr(Ord(ReceiveBuffer[(i - 1) * 4 + 1])) + ', ' +
                                                             IntToStr(Ord(ReceiveBuffer[(i - 1) * 4 + 2])) + ', ' +
                                                             IntToStr(Ord(ReceiveBuffer[(i - 1) * 4 + 3])) + ', ' +
                                                             IntToStr(Ord(ReceiveBuffer[(i - 1) * 4 + 4])) + '.     0x' +
                                                             IntToHex(Ord(ReceiveBuffer[(i - 1) * 4 + 1]), 2) +
                                                             IntToHex(Ord(ReceiveBuffer[(i - 1) * 4 + 2]), 2) +
                                                             IntToHex(Ord(ReceiveBuffer[(i - 1) * 4 + 3]), 2) +
                                                             IntToHex(Ord(ReceiveBuffer[(i - 1) * 4 + 4]), 2)
                                                             );
  end;
  FlushReceiveBuffer(COMnumber);
end;

procedure TForm1.cmbComPortChange(Sender: TObject);
begin
  try
    CloseCom(COMnumber);
    COMnumber := cmbComPort.ItemIndex + 1;
    OpenCom(COMnumber, BaudRate_115K, Parity_none, 8, Stopbits_1, 61, 60);

    if not ComIsOpen(COMnumber) then
      StatusBar1.Panels.Items[0].Text := 'Can''t open COM port ' + IntToStr(COMnumber)
    else
      StatusBar1.Panels.Items[0].Text := 'COM port ' + IntToStr(COMnumber) + ' was open successfully';
  except

  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  try
    SaveSettingsToIni;
    CloseCom(COMnumber);
  except
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  tmrStartup.Enabled := True;
end;

procedure TForm1.tmrStartupTimer(Sender: TObject);
begin
  tmrStartup.Enabled := False;

  if not ComIsOpen(COMnumber) then
  begin
    CurrentFileName := ExtractFilePath(ParamStr(0)) + 'lcd.bit';
    LoadSettingsFromIni;
    cmbComPort.ItemIndex := COMnumber - 1;
  end;

  CloseCom(COMnumber);

  StatusBar1.Panels.Items[0].Text := 'Opening COM port ' + IntToStr(COMnumber);
  OpenCom(COMnumber, BaudRate_115K, Parity_none, 8, Stopbits_1, 61, 60);

  if not ComIsOpen(COMnumber) then
  begin
    tmrStartup.Enabled := True;
    tmrStartup.Interval := 1000;
    StatusBar1.Panels.Items[0].Text := 'Can''t open COM port ' + IntToStr(COMnumber);
  end
  else
    StatusBar1.Panels.Items[0].Text := 'COM port ' + IntToStr(COMnumber) + ' was open successfully';
end;

procedure TForm1.LoadSettingsFromIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGCDCLoader.ini');
  try
    COMnumber := IniFile.ReadInteger('COM port', 'Number', 1);
    CurrentFileName := IniFile.ReadString('Bit file', 'FileName', 'lcd.bit');
    StatusBar1.Panels.Items[1].Text := CurrentFileName;

    Left := IniFile.ReadInteger('Window', 'Left', Screen.Width div 2 - Width div 2);
    Top := IniFile.ReadInteger('Window', 'Top', Screen.Height div 2 - Height div 2);
    Width := IniFile.ReadInteger('Window', 'Width', Width);
    Height := IniFile.ReadInteger('Window', 'Height', Height);

  finally

  end;
end;

procedure TForm1.SaveSettingsToIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGCDCLoader.ini');
  try
    IniFile.WriteInteger('COM port', 'Number', COMnumber);
    IniFile.WriteString('Bit file', 'FileName', CurrentFileName);

    IniFile.WriteInteger('Window', 'Left', Left);
    IniFile.WriteInteger('Window', 'Top', Top);
    IniFile.WriteInteger('Window', 'Width', Width);
    IniFile.WriteInteger('Window', 'Height', Height);
  finally

  end;
end;

end.
