unit PicUsbHid;

// - VCC -
// - 2009 -

{USB HID - List of devices}
{Readonly devices are not displayed}

interface

uses
  Windows, SysUtils, Classes, Messages, Forms, ExtCtrls;

{The FTempListOfDevices list is compared against FListOfDevices resulting in
 FListOfDevices update.
 This update must call OnHIDConnected and OnHIDDisconnected.}

const
  CUpdateHidListInterval = 5000;
  CPeriodicallyUpdateHidList = False;

type
  {Device data record}
  THIDDevice = packed record
    VID: Word;
    PID: Word;
    Name: string;
    VersionNumber: Word;
    SerialNumber: string;
    DevicePath: string;
  end;

  TArrayOfTHIDDevice = array of THIDDevice;
  TArrStr = array of string;

  TOnHIDConnected = procedure(Sender: TObject; Device: THIDDevice) of object;

  TOnHIDDisconnected = procedure(Sender: TObject; Device: THIDDevice) of object;

  TOnRedirectedWndProc = procedure(Sender: TObject; var Msg: TMessage) of object;

  TPicUsbHid = class(TComponent)
  private
    { Private declarations }
    FRequiredLength: DWORD;
    FOldWindowProc: TWndMethod;
    FHIDCount: Longint;
    FPeriodicallyUpdateHidList: Boolean;
    FUpdateHidListInterval: Integer;

    FtmrUpdate: TTimer;

    FListOfDevices: TArrayOfTHIDDevice;
    FTempListOfDevices: TArrayOfTHIDDevice; {Temporary list}
    FListOfNewConnected: TArrayOfTHIDDevice;  {also temporary}
    FListOfNewDisconnected: TArrayOfTHIDDevice; {also temporary}

    FFormThatReceivesMessages: TForm;

    FOnHIDConnected: TOnHIDConnected;
    FOnHIDDisconnected: TOnHIDDisconnected;
    FOnRedirectedWndProc: TOnRedirectedWndProc;
    
    procedure DoOnHIDConnected(Sender: TObject; Device: THIDDevice);
    procedure DoOnHIDDisconnected(Sender: TObject; Device: THIDDevice);
    procedure DoOnRedirectedWndProc(Sender: TObject; var Msg: TMessage);

    procedure SetFormThatReceivesMessages(Value: TForm);
    procedure SetPeriodicallyUpdateHidList(Value: Boolean);
    procedure SetUpdateHidListInterval(Value: Integer);


    procedure tmrUpdateTimer(Sender: TObject);

    procedure RedirectFromWMDeviceChange(var Msg: TMessage); 
    procedure RedirectFromWndProc(var Msg: TMessage);
    
    procedure AddDeviceToList(Device: THIDDevice);
    procedure AddDeviceToTempList(Device: THIDDevice);
    procedure UpdateDeviceList;
    procedure CompareLists;
    procedure CreateListOfNewConnectedDevices;
    procedure CreateListOfNewDisconnectedDevices;
    procedure AddDeviceToNewConnectedList(Device: THIDDevice);
    procedure AddDeviceToNewDisconnectedList(Device: THIDDevice);

    procedure MakeHidList;
    procedure RequestDetails(ListOfFiles: TArrStr);

  protected
    { Protected declarations }
    function GetListHidCount: Integer;         {for debug: move this to public}
    function GetTempListHidCount: Integer;     {for debug: move this to public}
    function DeviceIsInTempListByVidPid(Vid, Pid: Word): Boolean; {for debug: move this to public}
    function DeviceIsInTempListByDevice(Device: THIDDevice; CompareOSpath: Boolean = True): Boolean;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function DeviceIsInListByDevice(Device: THIDDevice; CompareOSpath: Boolean = True): Boolean;
    function DeviceIsInListByVidPid(Vid, Pid: Word): Boolean;
    function GetHIDDeviceByIndex(i: Integer): THIDDevice;
    function TotalHidCount: Integer;
    
  published
    { Published declarations }
    property FormThatReceivesMessages: TForm read FFormThatReceivesMessages
      write SetFormThatReceivesMessages default nil;
    property PeriodicallyUpdateHidList: Boolean read FPeriodicallyUpdateHidList
      write SetPeriodicallyUpdateHidList default CPeriodicallyUpdateHidList;
    property UpdateHidListInterval: Integer read FUpdateHidListInterval write
      SetUpdateHidListInterval default CUpdateHidListInterval;


    property OnHIDConnected: TOnHIDConnected read FOnHIDConnected write
      FOnHIDConnected;
    property OnHIDDisconnected: TOnHIDDisconnected read FOnHIDDisconnected write
      FOnHIDDisconnected;
    property OnRedirectedWndProc: TOnRedirectedWndProc read FOnRedirectedWndProc
      write FOnRedirectedWndProc;
  end;


const
  DIGCF_PRESENT         =  $00000002;
  DIGCF_DEVICEINTERFACE =  $00000010;

type
  HDEVINFO = Pointer;
  lptr = ^DWORD;

  PSP_DEVINFO_DATA = ^SP_DEVINFO_DATA;
  SP_DEVINFO_DATA = record
    cbSize: DWORD;
    ClassGuid: TGUID;
    DevInst: DWORD;
    Reserved: DWORD;
  end;

  PSP_DEVICE_INTERFACE_DATA = ^SP_DEVICE_INTERFACE_DATA;
  SP_DEVICE_INTERFACE_DATA = record
    cbSize: DWORD;
    InterfaceClassGuid: TGUID;
    Flags: DWORD;
    Reserved: DWORD;
  end;

  PSP_DEVICE_INTERFACE_DETAIL_DATA = ^SP_DEVICE_INTERFACE_DETAIL_DATA;
  SP_DEVICE_INTERFACE_DETAIL_DATA = packed record
    cbSize: DWORD;
    DevicePath: array [0..0] of AnsiChar;
  end;

  PHID_ATTRIBUTES = ^HID_ATTRIBUTES;
  HID_ATTRIBUTES = record
    Size: DWORD;
    VendorID: Word;
    ProductID: Word;
    VersionNumber: Word;
  end;

function SetupDiGetClassDevs(ClassGuid: PGUID; Enumerator: PChar;
  hwndParent: HWND; Flags: longword): HDEVINFO; stdcall;

function SetupDiEnumDeviceInterfaces(DeviceInfoSet: HDEVINFO;
  DeviceInfoData: PSP_DEVINFO_DATA; InterfaceClassGuid: PGUID;
  MemberIndex: DWORD; DeviceInterfaceData:
  PSP_DEVICE_INTERFACE_DATA): Boolean; stdcall;

function SetupDiGetDeviceInterfaceDetail(DeviceInfoSet: HDEVINFO;
  DeviceInterfaceData: PSP_DEVICE_INTERFACE_DATA;
  DeviceInterfaceDetailData: PSP_DEVICE_INTERFACE_DETAIL_DATA;
  DeviceInterfaceDetailDataSize: DWORD; RequiredSize: lptr;
  DeviceInfoData: PSP_DEVINFO_DATA): Boolean; stdcall;

function SetupDiDestroyDeviceInfoList(DeviceInfoSet: HDEVINFO): Boolean; stdcall;

procedure HidD_GetHidGuid(HidGuid: PGUID) stdcall;

function HidD_GetAttributes(HidDeviceObject: THandle;
  Attributes: PHID_ATTRIBUTES): Boolean; stdcall;

function HidD_GetProductString(HidDeviceObject: THandle; Buffer: PWideChar;
  BufferLength: Integer): Boolean; stdcall;

function HidD_GetSerialNumberString(HidDeviceObject: THandle; Buffer: PWideChar;
  BufferLength: Integer): Boolean; stdcall;

procedure Register;

implementation

{-$R PicUsbHid.res}

procedure Register;
begin
  RegisterComponents('Samples', [TPicUsbHid]);
end;



function SetupDiGetClassDevs; external 'SetupAPI.dll'
  name 'SetupDiGetClassDevsA';

function SetupDiEnumDeviceInterfaces; external 'SetupAPI.dll'
  name 'SetupDiEnumDeviceInterfaces';

function SetupDiGetDeviceInterfaceDetail; external 'SetupAPI.dll'
  name 'SetupDiGetDeviceInterfaceDetailA';

function SetupDiDestroyDeviceInfoList; external 'SetupAPI.dll'
  name 'SetupDiDestroyDeviceInfoList';


procedure HidD_GetHidGuid; external 'hid.dll' name 'HidD_GetHidGuid';

function HidD_GetAttributes; external 'hid.dll' name 'HidD_GetAttributes';

function HidD_GetProductString; external 'hid.dll' name 'HidD_GetProductString';

function HidD_GetSerialNumberString; external 'hid.dll'
  name 'HidD_GetSerialNumberString';




{ TPicUsbHid }  

constructor TPicUsbHid.Create(AOwner: TComponent);
var
  SearchForForm: TComponent;
begin
  inherited Create(AOwner);

  FPeriodicallyUpdateHidList := CPeriodicallyUpdateHidList;
  FUpdateHidListInterval := CUpdateHidListInterval;
  
  SetLength(FListOfDevices, 0);

  if Owner is TForm then
    FFormThatReceivesMessages := Owner as TForm
  else
  begin
    SearchForForm := Owner;
    try
      while not (SearchForForm is TForm) and (SearchForForm <> nil) do
        SearchForForm := SearchForForm.Owner;

      if SearchForForm is TForm then
        FFormThatReceivesMessages := SearchForForm as TForm
      else
        FFormThatReceivesMessages := nil;
    except
      FFormThatReceivesMessages := nil;
    end;    
  end;

  FtmrUpdate := TTimer.Create(Self);
  FtmrUpdate.OnTimer := tmrUpdateTimer;
  FtmrUpdate.Interval := FUpdateHidListInterval;
  FtmrUpdate.Enabled := CPeriodicallyUpdateHidList;
end;

destructor TPicUsbHid.Destroy;
begin
  inherited Destroy;
end;


procedure TPicUsbHid.SetFormThatReceivesMessages(Value: TForm);
begin
  if FFormThatReceivesMessages <> Value then
    FFormThatReceivesMessages := Value;

  if FFormThatReceivesMessages is TForm then
  begin
    FOldWindowProc := FFormThatReceivesMessages.WindowProc; {save the original}
    FFormThatReceivesMessages.WindowProc := RedirectFromWndProc; {redirected}
  end;  
end;


procedure TPicUsbHid.RequestDetails(ListOfFiles: TArrStr);
var
  Buffer: array [0..255] of WideChar;
  Attributes: PHID_ATTRIBUTES;
  Device: THIDDevice;
  FHIDDeviceFile: THandle;      {for retrieving info}
  n, i: Integer;
  FName_Dev: string;
begin
  try
    n := Length(ListOfFiles);

    if n = 0 then
      Exit;
    
    for i := 0 to n - 1 do
    begin
      //clear data
      Device.VID := 0;
      Device.PID := 0;
      Device.Name := '';
      Device.VersionNumber := 0;
      Device.DevicePath := '';
      Device.SerialNumber := '';

      FHIDDeviceFile := CreateFile(PChar(ListOfFiles[i]),
                                   GENERIC_READ or GENERIC_WRITE,
                                   FILE_SHARE_READ or FILE_SHARE_WRITE,
                                   nil,
                                   OPEN_EXISTING,
                                   0,
                                   0);

      GetMem(Attributes, SizeOf(HID_ATTRIBUTES));
      Attributes^.Size := SizeOf(HID_ATTRIBUTES);

      HidD_GetAttributes(FHIDDeviceFile, Attributes); {may fail for readonly devices like mice because of HID_ATTRIBUTES}
    
      if HidD_GetProductString(FHIDDeviceFile, Buffer, SizeOf(Buffer)) then
        FName_Dev := string(Buffer)
      else
        FName_Dev := 'No name';

      if HidD_GetSerialNumberString(FHIDDeviceFile, Buffer, SizeOf(Buffer)) then
        Device.SerialNumber := string(Buffer)
      else
        Device.SerialNumber := '';

      Device.VID := Attributes^.VendorID;
      Device.PID := Attributes^.ProductID;
      Device.Name := FName_Dev;
      Device.VersionNumber := Attributes^.VersionNumber;
      Device.DevicePath := ListOfFiles[i];

      FileClose(FHIDDeviceFile);
      FreeMem(Attributes);

      AddDeviceToTempList(Device);
    end; //for

  except
  end;
end;

procedure TPicUsbHid.RedirectFromWMDeviceChange(var Msg: TMessage);
begin
  DoOnRedirectedWndProc(Self, Msg);
  MakeHidList;
end;

procedure TPicUsbHid.MakeHidList;
var
  ClassDeviceData: PSP_DEVICE_INTERFACE_DETAIL_DATA;
  deviceInfoData: SP_DEVICE_INTERFACE_DATA;
  hidGuid: TGUID;
  AllHIDDeviceInfo: HDEVINFO;
  pchPath: PAnsiChar;
  Apth: TArrStr;  {array of paths}
  n: Integer;
begin
  Finalize(FTempListOfDevices);
  Finalize(Apth);

  HidD_GetHidGuid(@hidGuid);
  AllHIDDeviceInfo := SetupDiGetClassDevs(@hidGuid,
                                          nil,
                                          0,
                                          DIGCF_PRESENT or
                                            DIGCF_DEVICEINTERFACE);
  FHIDCount := 0;      // The first HID device
  deviceInfoData.cbSize := SizeOf(deviceInfoData);

  while True do     
  begin
    if not (SetupDiEnumDeviceInterfaces(AllHIDDeviceInfo,
                                        nil,
                                        @hidGuid,
                                        FHIDCount,
                                        @deviceInfoData)) then
      Break
    else
    begin
      // There is at least one HID device containing "ClassDeviceData"
      SetupDiGetDeviceInterfaceDetail(AllHIDDeviceInfo, //call to get size
                                      @deviceInfoData, 
                                      nil,
                                      0,
                                      @FRequiredLength,
                                      nil);
                                      
      GetMem(ClassDeviceData, FRequiredLength);
      ClassDeviceData^.cbSize := SizeOf(SP_DEVICE_INTERFACE_DETAIL_DATA);
      if not SetupDiGetDeviceInterfaceDetail(AllHIDDeviceInfo,
                                             @deviceInfoData,
                                             ClassDeviceData,
                                             FRequiredLength,
                                             @FRequiredLength,
                                             nil) then
      begin  {something wrong - can't get devices}
        FreeMem(ClassDeviceData);
	      Break;
      end;

      pchPath := ClassDeviceData.DevicePath;
      n := Length(Apth);
      SetLength(Apth, n + 1);
      Apth[n] := string(pchPath);

      ClassDeviceData.DevicePath := '1'; {some text to clear}
      FreeMem(ClassDeviceData);
      
      Inc(FHIDCount);    {The number of enumerated HID devices}
    end; //else begin  - found some devices
  end; //while

  try
    SetupDiDestroyDeviceInfoList(AllHIDDeviceInfo);
    try
      RequestDetails(Apth);
    finally
      Finalize(Apth);
      CompareLists;
      UpdateDeviceList;
    end;  
  except
  end;
end;


procedure TPicUsbHid.RedirectFromWndProc(var Msg: TMessage);
begin
  if Msg.Msg = WM_DEVICECHANGE then
    RedirectFromWMDeviceChange(Msg);

  DoOnRedirectedWndProc(Self, Msg);

  if Assigned(FOldWindowProc) then
    FOldWindowProc(Msg); {Call the original WindowProc}
end;

procedure TPicUsbHid.DoOnHIDConnected(Sender: TObject; Device: THIDDevice);
begin
  if Assigned(OnHIDConnected) then
    OnHIDConnected(Sender, Device);
end;
    
procedure TPicUsbHid.DoOnHIDDisconnected(Sender: TObject; Device: THIDDevice);
begin
  if Assigned(OnHIDDisconnected) then
    OnHIDDisconnected(Sender, Device);
end;

procedure TPicUsbHid.DoOnRedirectedWndProc(Sender: TObject; var Msg: TMessage);
begin
  if Assigned(OnRedirectedWndProc)
    then OnRedirectedWndProc(Sender, Msg);
end;

function TPicUsbHid.TotalHidCount: Integer;
begin
  Result := FHIDCount;
end;

procedure TPicUsbHid.AddDeviceToList(Device: THIDDevice);
var
  n: Integer;
begin
  {if there are more devices with the same VID and PID, there is a problem}
  //if DeviceIsInListByVidPid(Device.VID, Device.PID) then

  {comment this if there are allowed two devices having the same info about them}
  {look at DeviceIsInListByDevice code before commenting}
  if DeviceIsInListByDevice(Device, False) then
    Exit;
  n := Length(FListOfDevices);
  SetLength(FListOfDevices, n + 1);
  FListOfDevices[n] := Device;
end;

procedure TPicUsbHid.AddDeviceToTempList(Device: THIDDevice);
var
  n: Integer;
begin
  {if there are more devices with the same VID and PID, there is a problem}
  //if DeviceIsInTempListByVidPid(Device.VID, Device.PID) then

  {comment this if there are allowed two devices having the same info about them}
  {look at DeviceIsInTempListByDevice code before commenting}
  if DeviceIsInTempListByDevice(Device, False) then
    Exit;
  n := Length(FTempListOfDevices);
  SetLength(FTempListOfDevices, n + 1);
  FTempListOfDevices[n] := Device;
end;

function TPicUsbHid.DeviceIsInListByDevice(Device: THIDDevice; CompareOSpath: Boolean = True): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Length(FListOfDevices) - 1 do
    if (FListOfDevices[i].VID = Device.Vid) and
      (FListOfDevices[i].PID = Device.Pid) and
      (FListOfDevices[i].Name = Device.Name) and
      (FListOfDevices[i].VersionNumber = Device.VersionNumber) and
      (FListOfDevices[i].SerialNumber = Device.SerialNumber) then
      if CompareOSpath then
        if (FListOfDevices[i].DevicePath = Device.DevicePath) then
        begin
          Result := True;
          Break;
        end
        else
      else
      begin
        Result := True;
        Break;
      end
end;

function TPicUsbHid.DeviceIsInTempListByDevice(Device: THIDDevice; CompareOSpath: Boolean = True): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Length(FTempListOfDevices) - 1 do
    if (FTempListOfDevices[i].VID = Device.Vid) and
      (FTempListOfDevices[i].PID = Device.Pid) and
      (FTempListOfDevices[i].Name = Device.Name) and
      (FTempListOfDevices[i].VersionNumber = Device.VersionNumber) and
      (FTempListOfDevices[i].SerialNumber = Device.SerialNumber) then
      if CompareOSpath then
        if (FTempListOfDevices[i].DevicePath = Device.DevicePath) then
        begin
          Result := True;
          Break;
        end
        else
      else
      begin
        Result := True;
        Break;
      end 
end;

function TPicUsbHid.DeviceIsInListByVidPid(Vid, Pid: Word): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Length(FListOfDevices) - 1 do
    if (FListOfDevices[i].VID = Vid) and (FListOfDevices[i].PID = Pid) then
    begin
      Result := True;
      Break;
    end;  
end;

function TPicUsbHid.DeviceIsInTempListByVidPid(Vid, Pid: Word): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Length(FTempListOfDevices) - 1 do
    if (FTempListOfDevices[i].VID = Vid) and
      (FTempListOfDevices[i].PID = Pid) then
    begin
      Result := True;
      Break;
    end;  
end;

procedure TPicUsbHid.UpdateDeviceList;
var
  i: Integer;
begin
  Finalize(FListOfDevices);
  for i := 0 to Length(FTempListOfDevices) - 1 do
    AddDeviceToList(FTempListOfDevices[i]);
end;


procedure TPicUsbHid.CompareLists;
begin
  CreateListOfNewConnectedDevices;
  CreateListOfNewDisconnectedDevices;
end;

procedure TPicUsbHid.CreateListOfNewConnectedDevices;
var
  i: Integer;
begin   {devices that exist in temp list but do not exist in old list}
  Finalize(FListOfNewConnected);
  for i := 0 to Length(FTempListOfDevices) - 1 do
    if not DeviceIsInListByVidPid(FTempListOfDevices[i].VID,
                                  FTempListOfDevices[i].PID) then
    begin
      AddDeviceToNewConnectedList(FTempListOfDevices[i]);
      try
        DoOnHIDConnected(Self, FTempListOfDevices[i]);
      except
      end;
    end;
end;

procedure TPicUsbHid.CreateListOfNewDisconnectedDevices;
var
  i: Integer;
begin   {devices that exist in old list but do not exist in temp list}
  Finalize(FListOfNewDisconnected);
  for i := 0 to Length(FListOfDevices) - 1 do
    if not DeviceIsInTempListByVidPid(FListOfDevices[i].VID,
                                      FListOfDevices[i].PID) then
    begin
      AddDeviceToNewDisconnectedList(FListOfDevices[i]);
      try
        DoOnHIDDisconnected(Self, FListOfDevices[i]);
      except
      end;
    end;  
end;

procedure TPicUsbHid.AddDeviceToNewConnectedList(Device: THIDDevice);
var
  n: Integer;
begin
  n := Length(FListOfNewConnected);
  SetLength(FListOfNewConnected, n + 1);
  FListOfNewConnected[n] := Device;
end;

procedure TPicUsbHid.AddDeviceToNewDisconnectedList(Device: THIDDevice);
var
  n: Integer;
begin
  n := Length(FListOfNewDisconnected);
  SetLength(FListOfNewDisconnected, n + 1);
  FListOfNewDisconnected[n] := Device;
end;

function TPicUsbHid.GetHIDDeviceByIndex(i: Integer): THIDDevice;
begin
  Result.VID := 0;
  Result.PID := 0;
  Result.Name := 'Error: Index ' + IntToStr(i) + ' exceeds the list count.';
  Result.VersionNumber := 0;
  Result.SerialNumber := '';
  Result.DevicePath := '';
  if i > Length(FListOfDevices) - 1 then
    Exit;
  Result := FListOfDevices[i];
end;

function TPicUsbHid.GetListHidCount: Integer;
begin
  Result := Length(FListOfDevices);
end;

function TPicUsbHid.GetTempListHidCount: Integer;
begin
  Result := Length(FTempListOfDevices);
end;

procedure TPicUsbHid.SetPeriodicallyUpdateHidList(Value: Boolean);
begin
  if FPeriodicallyUpdateHidList <> Value then
  begin
    FPeriodicallyUpdateHidList := Value;
    FtmrUpdate.Enabled := FPeriodicallyUpdateHidList;
  end;
end;

procedure TPicUsbHid.tmrUpdateTimer(Sender: TObject);
begin
  MakeHidList;
end;

procedure TPicUsbHid.SetUpdateHidListInterval(Value: Integer);
begin
  if FUpdateHidListInterval <> Value then
  begin
    FUpdateHidListInterval := Value;
    if Assigned(FtmrUpdate) then
      FtmrUpdate.Interval := Value;
  end;
end;

end.
 