unit Unit1;

// - VCC -
// Main window of JTAG HID Loader
// For Spartan-3(E) and Spartan-6 FPGAs
// - 2010 -
// - 2013 -

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, PicUsbHid;

type
  TReadFromPICTh = class(TThread)
  protected
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    btnGetJTAGChain: TButton;
    btnProgramSelectedDevice: TButton;
    tmrStartup: TTimer;
    lstLog: TListBox;
    btnAbortProgramming: TButton;
    prbProgramming: TProgressBar;
    pnlProgrammingStatus: TPanel;
    lbeDeviceToProgram: TLabeledEdit;
    pnlProgramPos: TPanel;
    StatusBar1: TStatusBar;
    btnBrowseForBitfile: TButton;
    OpenDialog1: TOpenDialog;
    btnBlinkLED: TButton;
    btnGetIDCodes: TButton;
    procedure btnGetJTAGChainClick(Sender: TObject);
    procedure tmrStartupTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnProgramSelectedDeviceClick(Sender: TObject);
    procedure btnAbortProgrammingClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnBrowseForBitfileClick(Sender: TObject);
    procedure PicUsbHid1HIDConnected(Sender: TObject; Device: THIDDevice);
    procedure PicUsbHid1HIDDisconnected(Sender: TObject; Device: THIDDevice);
    procedure PicUsbHid1RedirectedWndProc(Sender: TObject; var Msg: TMessage);
    procedure btnBlinkLEDClick(Sender: TObject);
    procedure btnGetIDCodesClick(Sender: TObject);
  private
    { Private declarations }
    FPicUsbHid: TPicUsbHid;
    PIC18F_In: THandle;
    PIC18F_Out: THandle;
    FDataBufferRD: array[0..64] of Byte;
    FDataBufferWR: array[0..64] of Byte;
    FPICisConnected: Boolean;

    FReadingThread: TReadFromPICTh;
    procedure OnReadFromPICThTerminate(Sender: TObject);

  public
    { Public declarations }
    function peek: Boolean; {Peek Message}
    procedure GetJTAGChain;
    procedure ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
    procedure LoadSettingsFromIni;
    procedure SaveSettingsToIni;
    procedure FlushTheReceiveBuffer;
    procedure DisconnectFromPIC;
    procedure ReadFromPIC;
  end;

const
  timeout = 2000;


  CMaxDevices = 30; {Maximum number of devices}
  CDBDeviceCount = 19; {Number of items in Device database}

  CDeviceNameArray: array[0..CDBDeviceCount] of AnsiString =
    ('XC3S100E', 'XC3S250E', 'XC3S500E', 'XC3S1200E', 'XC3S1600E',
    'XCF02S', 'XCF04S', 'XCF08P',
    'XC3S400',
    'XC2C64A',
    'XC6SLX4',
    'XC6SLX9',
    'XC6SLX16',
    'XC6SLX25',
    'XC6SLX25T',
    'XC6SLX45',
    'XC6SLX45T',
    'XC6SLX75',
    'XC6SLX75T',

    'Unknown');

    
  {define your VID and PID here}
  My_VID: Word = $04D8;
  My_PID: Word = $1234;

  {Report Length}
  ReadLen = 64;
  WriteLen = 64;  

var
  Form1: TForm1;
  ChainLength: Byte;
  AbortProgrammging: Boolean;

  CurrentFileName: TFileName;

implementation

{$R *.dfm}

uses
  JTAGdataProvider, BitToTibConverter, IniFiles;


procedure TReadFromPICTh.Execute;
begin
  try
    repeat
      //Synchronize(Form1.ReadFromPIC);
      Form1.ReadFromPIC;
      if Form1.FDataBufferRD[1] = 192 then
        Break;
    until Terminated;
  except

  end;
end;

procedure TForm1.OnReadFromPICThTerminate(Sender: TObject);
begin
  try
    if Assigned(FReadingThread) then
      FreeAndNil(FReadingThread);
  except

  end;
end;

function TForm1.peek: Boolean;
var
  msg: TMsg;
begin
  Result := False;
  if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then
  begin
    if (msg.message = WM_QUIT) then
      Result := True
    else
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end
  end;
end;

procedure TForm1.GetJTAGChain;
const
  CTimeout: Cardinal = 3000; //3s
var
  i: Byte;
  tk: Cardinal;
begin
  FDataBufferWR[0] := 0;
  FDataBufferWR[1] := 10;
  FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);
  tk := GetTickCount;
  repeat
    peek;
  until (FDataBufferRD[1] > 0) or (GetTickCount - tk >= CTimeout); {wait for PIC to answer}

  if GetTickCount - tk >= CTimeout then
    MessageBox(Handle, 'Can''t get JTAG chain.', PChar(Application.Title), MB_ICONERROR)
  else
  begin
    ChainLength := Ord(FDataBufferRD[1]);
    lstLog.Items.Add('');
    lstLog.Items.Add('ChainLength = ' + IntToStr(ChainLength));

    for i := 1 to ChainLength do
      lstLog.Items.Add(WideString(CDeviceNameArray[Ord(FDataBufferRD[i + 1])]) + '  Position: ' + IntToStr(i));
  end;
  FlushTheReceiveBuffer; 
end;

procedure TForm1.PicUsbHid1HIDConnected(Sender: TObject; Device: THIDDevice);
begin
  if (Device.VID = My_VID) and (Device.PID = My_PID) then
  begin
    FPICisConnected := True;
    PIC18F_In := CreateFile(PChar(Device.DevicePath),
                              GENERIC_READ,
                              FILE_SHARE_READ or FILE_SHARE_WRITE, {do not change this}
                              nil,
                              OPEN_EXISTING,
                              0,
                              0);
    PIC18F_Out := CreateFile(PChar(Device.DevicePath),
                               GENERIC_WRITE,
                               FILE_SHARE_READ or FILE_SHARE_WRITE, {do not change this}
                               nil,
                               OPEN_EXISTING,
                               0,
                               0);

    FReadingThread := TReadFromPICTh.Create(True);
    FReadingThread.FreeOnTerminate := True;
    FReadingThread.OnTerminate := OnReadFromPICThTerminate;
    FReadingThread.Resume;

    StatusBar1.Panels.Items[0].Text := 'Connected:  ' + Device.Name + ' VID: ' + IntToStr(Device.VID) + ' PID: ' + IntToStr(Device.PID);
  end;
end;

procedure TForm1.PicUsbHid1HIDDisconnected(Sender: TObject; Device: THIDDevice);
begin
  if (Device.VID = My_VID) and (Device.PID = My_PID) then
  begin
    FPICisConnected := False;
    if PIC18F_In > 0 then
      CloseHandle(PIC18F_In);
    if PIC18F_Out > 0 then
      CloseHandle(PIC18F_Out);

    StatusBar1.Panels.Items[0].Text := 'Disconnected:  ' + Device.Name + ' VID: ' + IntToStr(Device.VID) + ' PID: ' + IntToStr(Device.PID);

    try
      if Assigned(FReadingThread) then
        FReadingThread.Terminate;
    except

    end;
  end;
end;

procedure TForm1.PicUsbHid1RedirectedWndProc(Sender: TObject; var Msg: TMessage);
begin
  // If you want to process some messages, put your code here
end;

procedure TForm1.ProgramFPGA(ATibFile: TFileName; PositionInChain: Byte);
var
  NoMoreData: Boolean;
  arr: ArrayOf61Chars;
  DataLength, i: Byte;
begin
  OpenTheTibFile(ATibFile);

  FDataBufferWR[0] := 0;
  FDataBufferWR[1] := 20;
  FDataBufferWR[2] := PositionInChain;
  FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);  {program command + first device}
  Sleep(1);

  prbProgramming.Position := 0;
  AbortProgrammging := False;
  repeat

    repeat
      peek;
      if FDataBufferRD[1] = 25 then   {PIC asks for more data}
      begin
        FlushTheReceiveBuffer;
        Break;
      end;
    until AbortProgrammging;
    
    ReadFromTibFileAsAStream(@arr);
    DataLength := arr[0];

    if AbortProgrammging then
      arr[0] := arr[0] or 64;

    FDataBufferWR[0] := 0;
    for i := 1 to arr[0] and $3F + 1 do   {for a full block, this is 1 to 61}
      FDataBufferWR[i] := arr[i - 1]; 
    FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);
      

    NoMoreData := arr[0] and 128 = 128;

    DataLength := DataLength and $7F;

    prbProgramming.Position := prbProgramming.Position + DataLength;
    if Random(4) = 1 then  {do not update every time}
      pnlProgramPos.Caption := 'Program pos: ' + IntToStr(prbProgramming.Position) + ' / ' + IntToStr(prbProgramming.Max);

  until NoMoreData or AbortProgrammging;

  CloseTheTibFile;

  pnlProgramPos.Caption := 'Program pos: ' + IntToStr(prbProgramming.Position) + ' / ' + IntToStr(prbProgramming.Max);
end;

procedure TForm1.btnGetJTAGChainClick(Sender: TObject);
begin
  if not FPICisConnected then
    Exit;

  GetJTAGChain;
  
  if ChainLength > 0 then
    btnProgramSelectedDevice.Enabled := True;

  lstLog.ItemIndex := lstLog.Count - 1; //Select last line    
end;

procedure TForm1.btnProgramSelectedDeviceClick(Sender: TObject);
var
  TibFileName: string;
  PositionToProgram: Byte;
  BitLength: Integer;
begin
  if not FPICisConnected then
    Exit;

  PositionToProgram := StrToInt(lbeDeviceToProgram.Text);

  if not FileExists(CurrentFileName) then
    if not OpenDialog1.Execute then
      Exit
    else
    begin
      CurrentFileName := OpenDialog1.FileName;
      StatusBar1.Panels.Items[1].Text := CurrentFileName;
    end;

  pnlProgrammingStatus.Caption := 'Programming';
  pnlProgrammingStatus.Repaint;

  btnGetJTAGChain.Enabled := False;
  btnProgramSelectedDevice.Enabled := False;


  try
    if PositionToProgram > ChainLength then
      MessageBox(Handle, PChar('Position selected exceeds chain length.' + #13#10 + 'Valid positions are in range 1..' + IntToStr(ChainLength)), 'JTAG error', MB_ICONERROR)
    else
    begin
      TibFileName := ConvertBitFileToTibFile(CurrentFileName, BitLength);
      prbProgramming.Max := BitLength;
      ProgramFPGA(TibFileName, PositionToProgram);  {Send the .tib file to microcontroller}
    end;
  finally
    btnGetJTAGChain.Enabled := True;
    btnProgramSelectedDevice.Enabled := True;
  end;

  pnlProgrammingStatus.Caption := 'Finished';
  pnlProgrammingStatus.Repaint;
end;

procedure TForm1.btnAbortProgrammingClick(Sender: TObject);
begin
  AbortProgrammging := True;
end;

procedure TForm1.btnBrowseForBitfileClick(Sender: TObject);
begin
  if not OpenDialog1.Execute then
    Exit;
    
  CurrentFileName := OpenDialog1.FileName;
  StatusBar1.Panels.Items[1].Text := CurrentFileName;
end;

procedure TForm1.btnBlinkLEDClick(Sender: TObject);
begin
  FDataBufferWR[0] := 0;
  FDataBufferWR[1] := 30;
  FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);
  Sleep(100);

  FlushTheReceiveBuffer;
end;

procedure TForm1.btnGetIDCodesClick(Sender: TObject);
const
  CTimeout: Cardinal = 3000;
var
  tk: Cardinal;
  i: Integer;
begin
  FDataBufferWR[0] := 0;
  FDataBufferWR[1] := 40;
  FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);
  Sleep(1);

  tk := GetTickCount;
  repeat
    peek;
  until (FDataBufferRD[1] > 0) or (GetTickCount - tk >= CTimeout); {wait for PIC to answer}

  if GetTickCount - tk >= CTimeout then
    MessageBox(Handle, 'Can''t get JTAG ID codes.', PChar(Application.Title), MB_ICONERROR)
  else
  begin
    ChainLength := Ord(FDataBufferRD[1]);
    lstLog.Items.Add('');
    lstLog.Items.Add('ChainLength = ' + IntToStr(ChainLength));

    for i := 1 to ChainLength do
      lstLog.Items.Add('Device ' + IntToStr(i) + '  ID = ' + IntToStr(Ord(FDataBufferRD[(i - 1) * 4 + 1])) + ', ' +
                                                             IntToStr(Ord(FDataBufferRD[(i - 1) * 4 + 2])) + ', ' +
                                                             IntToStr(Ord(FDataBufferRD[(i - 1) * 4 + 3])) + ', ' +
                                                             IntToStr(Ord(FDataBufferRD[(i - 1) * 4 + 4])) + '.     0x' +
                                                             IntToHex(Ord(FDataBufferRD[(i - 1) * 4 + 1]), 2) +
                                                             IntToHex(Ord(FDataBufferRD[(i - 1) * 4 + 2]), 2) +
                                                             IntToHex(Ord(FDataBufferRD[(i - 1) * 4 + 3]), 2) +
                                                             IntToHex(Ord(FDataBufferRD[(i - 1) * 4 + 4]), 2)
                                                             );
  end;
  FlushTheReceiveBuffer; 
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  try
    SaveSettingsToIni;
    DisconnectFromPIC;
  except
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FPICisConnected := False;
  StatusBar1.Panels.Items[0].Text := 'Not connected';
  tmrStartup.Enabled := True;
  FPicUsbHid := TPicUsbHid.Create(Form1);
  FPicUsbHid.OnHIDConnected := PicUsbHid1HIDConnected;
  FPicUsbHid.OnHIDDisconnected := PicUsbHid1HIDDisconnected;
  FPicUsbHid.FormThatReceivesMessages := Self;
  FPicUsbHid.OnRedirectedWndProc := PicUsbHid1RedirectedWndProc;
end;

procedure TForm1.tmrStartupTimer(Sender: TObject);
begin
  tmrStartup.Enabled := False;

  if not FPICisConnected then
  begin
    CurrentFileName := ExtractFilePath(ParamStr(0)) + 'lcd.bit';
    LoadSettingsFromIni;
  end;

  SendMessage(Handle, WM_DEVICECHANGE, 0, 0);
end;

procedure TForm1.LoadSettingsFromIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGHIDLoader.ini');
  try
    CurrentFileName := IniFile.ReadString('Bit file', 'FileName', 'lcd.bit');
    StatusBar1.Panels.Items[1].Text := CurrentFileName;

    Left := IniFile.ReadInteger('Window', 'Left', Screen.Width div 2 - Width div 2);
    Top := IniFile.ReadInteger('Window', 'Top', Screen.Height div 2 - Height div 2);
    Width := IniFile.ReadInteger('Window', 'Width', Width);
    Height := IniFile.ReadInteger('Window', 'Height', Height);

  finally

  end;
end;

procedure TForm1.SaveSettingsToIni;
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'JTAGHIDLoader.ini');
  try
    IniFile.WriteString('Bit file', 'FileName', CurrentFileName);

    IniFile.WriteInteger('Window', 'Left', Left);
    IniFile.WriteInteger('Window', 'Top', Top);
    IniFile.WriteInteger('Window', 'Width', Width);
    IniFile.WriteInteger('Window', 'Height', Height);
  finally

  end;
end;

procedure TForm1.FlushTheReceiveBuffer;
begin
  FDataBufferRD[1] := 0; {no need to clear other bytes, this is the command byte}
end;

{The reading thread is in a loop waiting for PIC to send data. To close this loop,
the computer has to send a disconnect command to PIC. As a response, the PIC sends
to PC a command that will terminate the thread.}
procedure TForm1.DisconnectFromPIC;
const
  TimeOut: Cardinal = 2000;
var
  tk: Cardinal;
begin
  if not FPICisConnected then
    Exit;
    
  FDataBufferWR[0] := 0;
  FDataBufferWR[1] := 200;  {disconnect command}
  FileWrite(PIC18F_Out, FDataBufferWR, WriteLen + 1);  {program command + first device}

  tk := GetTickCount;
  repeat
    {Meanwhile, the PIC should send the disconnect command (i.e. 192) to computer. This is processed in the reading thread.}
  until (FDataBufferRD[1] = 192) or (GetTickCount - tk >= TimeOut);
  FlushTheReceiveBuffer;
  FPICisConnected := False;
end;

procedure TForm1.ReadFromPIC;
begin   {blocking call}
  FileRead(PIC18F_In, FDataBufferRD, ReadLen + 1);
end;

end.
